@@ -1,7 +1,7 @@
-"""
-Create an SBML Report from given SBML file or set of SBML files (for instance for comp models).
+"""SBML report.
 
-The model report is implemented based on a standard template language,
+Create an SBML report from given SBML file or set of SBML files (for instance for comp
+models). The model report is implemented based on a standard template language,
 which uses the SBML information to render the final document.
 
 The basic steps of template creation are
@@ -9,14 +9,14 @@ The basic steps of template creation are
 - compile template
 - render with SBML context
 
-The final report consists of an HTML file with an overview over the SBML elements in the model.
+The final report consists of an HTML file with an overview over the SBML elements in
+the model.
 """
-import codecs
 import logging
 import ntpath
 import warnings
 from pathlib import Path
-from typing import Iterable, List
+from typing import Dict, Iterable, List
 
 import jinja2
 import libsbml
@@ -24,30 +24,42 @@ from jinja2 import TemplateNotFound
 
 from sbmlutils import RESOURCES_DIR, utils
 from sbmlutils.io.sbml import read_sbml, write_sbml
-from sbmlutils.report import formating, sbmlfilters
+from sbmlutils.report import formating, mathml, sbmlfilters
 
 
 logger = logging.getLogger(__name__)
 
-TEMPLATE_DIR = RESOURCES_DIR / "templates"  # template location
+TEMPLATE_DIR = RESOURCES_DIR / "templates"
 
 
 def create_reports(
     sbml_paths: Iterable[Path],
     output_dir: Path,
-    template="report.html",
-    promote=False,
-    validate=True,
+    template: str = "report.html",
+    promote: bool = False,
+    validate: bool = True,
+    math_type: str = "cmathml",
 ):
-    """Creates individual reports and an overview file.
+    """Create individual reports and an overview file.
 
     :param sbml_paths: paths to SBML files
-    :param output_dir:
-    :param template:
-    :param promote:
-    :param validate:
+    :param output_dir: target directory where the report is written
+    :param template: which template file to use for rendering
+    :param promote: boolean flag to promote local parameters
+    :param validate: boolean flag if SBML file be validated (warnings and errors
+                     are logged)
+    :param math_type: specifies the math rendering mode for the report
+
     :return:
     """
+
+    # check math type
+    math_types = ["cmathml", "pmathml", "latex"]
+    if math_type not in math_types:
+        raise ValueError(
+            f"math_type '{math_type}' not in supported types: '{math_types}'"
+        )
+
     # individual reports
     for sbml_path in sbml_paths:
         logger.info(sbml_path)
@@ -57,15 +69,14 @@ def create_reports(
             template=template,
             promote=promote,
             validate=validate,
+            math_type=math_type,
         )
 
     # write index html (unicode)
     html = _create_index_html(sbml_paths)
     index_path = output_dir / "index.html"
-    # FIXME: is this still necessary in python 3?
-    f_index = codecs.open(index_path, encoding="utf-8", mode="w")
-    f_index.write(html)
-    f_index.close()
+    with open(index_path, "w", encoding="utf-8") as f_index:
+        f_index.write(html)
 
 
 def create_report(
@@ -73,12 +84,13 @@ def create_report(
     output_dir: Path,
     promote: bool = False,
     template: str = "report.html",
+    math_type: str = "cmathml",
     validate: bool = True,
     log_errors: bool = True,
     units_consistency: bool = True,
     modeling_practice: bool = True,
 ):
-    """Creates HTML report for SBML file.
+    """Create HTML report for SBML file.
 
     The SBML file can be validated during report generation.
     Local parameters can be promoted during report generation.
@@ -87,7 +99,9 @@ def create_report(
     :param output_dir: target directory where the report is written
     :param promote: boolean flag to promote local parameters
     :param template: which template file to use for rendering
-    :param validate: boolean flag if SBML file be validated (warnings and errors are logged)
+    :param math_type: specifies the math rendering mode for the report
+    :param validate: boolean flag if SBML file be validated (warnings and errors
+                     are logged)
     :param log_errors: boolean flag of errors should be logged
     :param units_consistency: boolean flag units consistency
     :param modeling_practice: boolean flag modeling practise
@@ -96,12 +110,14 @@ def create_report(
     """
     if not isinstance(sbml_path, Path):
         logger.warning(
-            f"All paths should be of type 'Path', but '{type(sbml_path)}' found for: {sbml_path}"
+            f"All paths should be of type 'Path', "
+            f"but '{type(sbml_path)}' found for: {sbml_path}"
         )
         sbml_path = Path(sbml_path)
     if not isinstance(output_dir, Path):
         logger.warning(
-            f"All paths should be of type 'Path', but '{type(output_dir)}' found for: {output_dir}"
+            f"All paths should be of type 'Path', "
+            f"but '{type(output_dir)}' found for: {output_dir}"
         )
         output_dir = Path(output_dir)
 
@@ -113,6 +129,13 @@ def create_report(
     if not output_dir.is_dir():
         raise IOError(f"'output_dir' is not a directory: '{output_dir}'")
 
+    # check math type
+    math_types = ["cmathml", "pmathml", "latex"]
+    if math_type not in math_types:
+        raise ValueError(
+            f"math_type '{math_type}' not in supported types: '{math_types}'"
+        )
+
     # read sbml
     doc = read_sbml(
         source=sbml_path,
@@ -128,13 +151,11 @@ def create_report(
     name = ".".join(basename.split(".")[:-1])
     write_sbml(doc, filepath=output_dir / basename)
 
-    # write html (unicode)
-    html = _create_html(doc, basename, html_template=template)
+    # write html
+    html = _create_html(doc, basename, html_template=template, math_type=math_type)
     path_html = output_dir / f"{name}.html"
-    # FIXME: ist this still necessary
-    f_html = codecs.open(path_html, encoding="utf-8", mode="w")
-    f_html.write(html)
-    f_html.close()
+    with open(path_html, "w", encoding="utf-8") as f_html:
+        f_html.write(html)
 
     logger.info(f"SBML report created: '{path_html.resolve()}'")
 
@@ -142,7 +163,15 @@ def create_report(
 def _create_index_html(
     sbml_paths: List[Path], html_template: str = "index.html", offline: bool = True
 ):
-    """Create index.html for given sbml_paths."""
+    """Create index.html for given sbml_paths.
+
+    :param sbml_paths: List of paths to SBML files
+    :param html_template: which template file to use for rendering
+    :param offline: to specify offline report generation for appropriate linking
+                    of stylesheet and script files
+
+    :return
+    """
 
     # template environment
     env = jinja2.Environment(
@@ -161,24 +190,32 @@ def _create_index_html(
         name = ".".join(tokens[:-1]) + ".html"
         sbml_links.append(name)
 
-    # Context
-    c = {
-        "offline": offline,
-        "sbml_basenames": sbml_basenames,
-        "sbml_links": sbml_links,
-    }
-    return template.render(c)
+    return template.render(
+        {
+            "offline": offline,
+            "sbml_basenames": sbml_basenames,
+            "sbml_links": sbml_links,
+        }
+    )
 
 
-def _create_html(doc, basename, html_template="report.html", offline=True):
+def _create_html(
+    doc: libsbml.SBMLDocument,
+    basename: str,
+    html_template: str = "report.html",
+    math_type: str = "cmathml",
+    offline: bool = True,
+):
     """Create HTML from SBML.
 
-    :param doc:
-    :type doc:
-    :param html_template:
-    :type html_template:
-    :return:
-    :rtype:
+    :param doc: SBML document for creating HTML report
+    :param basename: basename of SBML file path
+    :param html_template: which template file to use for rendering
+    :param math_type: specifies the math rendering mode for the report
+    :param offline: to specify offline report generation for appropriate linking of
+                    stylesheet and script files
+
+    :return: rendered HTML report template for the SBML document
     """
     # template environment
     env = jinja2.Environment(
@@ -200,7 +237,7 @@ def _create_html(doc, basename, html_template="report.html", offline=True):
 
         values = _create_value_dictionary(model)
 
-        # Context
+        # context
         c = {
             "offline": offline,
             "basename": basename,
@@ -210,24 +247,22 @@ def _create_html(doc, basename, html_template="report.html", offline=True):
             "model": model_dict(model),
             "submodels": listOfSubmodels_dict(model),
             "ports": listOfPorts_dict(model),
-            "functions": listOfFunctions_dict(model),
+            "functions": listOfFunctions_dict(model, math_type),
             "units": listOfUnits_dict(model),
-            "compartments": listOfCompartments_dict(model, values),
+            "compartments": listOfCompartments_dict(model, values, math_type),
             "species": listOfSpecies_dict(model),
             "geneproducts": listOfGeneProducts_dict(model),
-            "parameters": listOfParameters_dict(model, values),
-            "assignments": listOfInitialAssignments_dict(model),
-            "rules": listOfRules_dict(model),
-            "reactions": listOfReactions_dict(model),
+            "parameters": listOfParameters_dict(model, values, math_type),
+            "assignments": listOfInitialAssignments_dict(model, math_type),
+            "rules": listOfRules_dict(model, math_type),
+            "reactions": listOfReactions_dict(model, math_type),
             "objectives": listOfObjectives_dict(model),
-            "constraints": listOfConstraints_dict(model),
-            "events": listOfEvents_dict(model),
+            "constraints": listOfConstraints_dict(model, math_type),
+            "events": listOfEvents_dict(model, math_type),
         }
     else:
         # no model exists
-        logging.error(
-            "No model in SBML file when creating model report: {}".format(doc)
-        )
+        logging.error(f"No model in SBML file when creating model report: {doc}")
         template = env.get_template("report_no_model.html")
         c = {
             "basename": basename,
@@ -236,12 +271,15 @@ def _create_html(doc, basename, html_template="report.html", offline=True):
     return template.render(c)
 
 
-##############################
+# ------------------------
 # Information Dictionaries
-##############################
+# ------------------------
+def _create_value_dictionary(model: libsbml.Model) -> Dict:
+    """Create dictionary of values for model instance.
 
-
-def _create_value_dictionary(model):
+    :param model: SBML model instance for which values dictionary is to be created
+    :return: values dictionary for model
+    """
     values = dict()
 
     # parse all the initial assignments
@@ -256,8 +294,12 @@ def _create_value_dictionary(model):
     return values
 
 
-def infoSbase(item):
-    """ Info dictionary for SBase. """
+def infoSbase(item: libsbml.SBase) -> Dict:
+    """Info dictionary for SBase.
+
+    :param item: SBase instance for which info dictionary is to be created
+    :return info dictionary for item
+    """
 
     info = {
         "object": item,
@@ -285,9 +327,10 @@ def infoSbase(item):
             submodel_ref = replaced_by.getSubmodelRef()
             info[
                 "replaced_by"
-            ] = '<br /><i class="fa fa-arrow-circle-right" aria-hidden="true"></i><code>ReplacedBy {}:{}</code>'.format(
-                submodel_ref, sbaseref(replaced_by)
-            )
+            ] = f"""
+                <br /><i class="fa fa-arrow-circle-right" aria-hidden="true"></i>
+                <code>ReplacedBy {submodel_ref}:{sbaseref(replaced_by)}</code>
+                """
 
         # ListOfReplacedElements
         if item_comp.getNumReplacedElements() > 0:
@@ -295,9 +338,10 @@ def infoSbase(item):
             for rep_el in item_comp.getListOfReplacedElements():
                 submodel_ref = rep_el.getSubmodelRef()
                 replaced_elements.append(
-                    '<br /><i class="fa fa-arrow-circle-left" aria-hidden="true"></i><code>ReplacedElement {}:{}</code>'.format(
-                        submodel_ref, sbaseref(rep_el)
-                    )
+                    f"""
+                    <br /><i class="fa fa-arrow-circle-left" aria-hidden="true"></i>
+                    <code>ReplacedElement {submodel_ref}:{sbaseref(rep_el)}</code>
+                    """
                 )
             if len(replaced_elements) == 0:
                 replaced_elements = ""
@@ -308,30 +352,43 @@ def infoSbase(item):
     return info
 
 
-def document_dict(doc):
+def document_dict(doc: libsbml.SBMLDocument) -> Dict:
+    """Create Info dictonary for SBML document instance.
+
+    :param doc: SBML document for which info dictionary is to be created
+    :return: info dictionary for doc
+    """
+
     info = infoSbase(doc)
-    packages = [
-        '<span class="package">L{}V{}</span>'.format(doc.getLevel(), doc.getVersion())
-    ]
+    packages = [f'<span class="package">L{doc.getLevel()}V{doc.getVersion()}</span>']
 
     for k in range(doc.getNumPlugins()):
         plugin = doc.getPlugin(k)
         prefix = plugin.getPrefix()
         version = plugin.getPackageVersion()
-        packages.append('<span class="package">{}-V{}</span>'.format(prefix, version))
+        packages.append(f'<span class="package">{prefix}-V{version}</span>')
     info["packages"] = " ".join(packages)
     return info
 
 
-def model_dict(model):
+def model_dict(model: libsbml.Model) -> Dict:
+    """Create Info dictionary for SBML Model instance.
+
+    :param model: SBML model for which info dictionary is to be created
+    :return: info dictionary for model
+    """
     info = infoSbase(model)
     info["history"] = formating.modelHistoryToString(model.getModelHistory())
 
     return info
 
 
-def listOfModelDefinitions_dict(doc):
-    """ Information dicts for ExternalModelDefinitions and ModelDefinitions"""
+def listOfModelDefinitions_dict(doc: libsbml.SBMLDocument) -> List[Dict]:
+    """Information dicts for ExternalModelDefinitions and ModelDefinitions.
+
+    :param: doc: SBML document enclosing ExternalModelDefinitions and ModelDefinitions
+    :return: list of info dictionaries for ExternalModelDefinitions and ModelDefinitions
+    """
     items = []
     doc_comp = doc.getPlugin("comp")
     if doc_comp:
@@ -341,14 +398,20 @@ def listOfModelDefinitions_dict(doc):
             items.append(info)
         for item in doc_comp.getListOfExternalModelDefinitions():
             info = infoSbase(item)
-            info["type"] = type(item).__name__ + " (<code>source={}</code>)".format(
-                item.getSource()
+            info["type"] = (
+                type(item).__name__ + f" (<code>source={item.getSource}</code>)"
             )
             items.append(info)
     return items
 
 
-def listOfSubmodels_dict(model):
+def listOfSubmodels_dict(model: libsbml.Model) -> List[Dict]:
+    """Information dicts for Submodels.
+
+    :param: model: SBML model enclosing the Submodels
+    :return: list of info dictionaries for Submodels within model
+    """
+
     items = []
     model_comp = model.getPlugin("comp")
     if model_comp:
@@ -377,12 +440,13 @@ def listOfSubmodels_dict(model):
     return items
 
 
-def sbase_ref_dict(item):
-    """Information dictionary of SbaseRef
+def sbase_ref_dict(item: libsbml.SBaseRef) -> Dict:
+    """Information dictionary of SBaseRef.
 
-    :param sbase_ref:
-    :return:
+    :param item: SBaseRef instance for which information dictionary has to be created
+    :return: information dictionary for item
     """
+
     info = infoSbase(item)
     port_ref = ""
     if item.isSetPortRef():
@@ -401,13 +465,20 @@ def sbase_ref_dict(item):
         metaid_ref = item.getMetaIdRef()
     info["metaid_ref"] = metaid_ref
     element = item.getReferencedElement()
-    info["referenced_element"] = "<code>{}: {}</code>".format(
-        type(element).__name__, element.getId()
-    )
+    info[
+        "referenced_element"
+    ] = f"<code>{type(element).__name__}: {element.getId()}</code>"
     return info
 
 
-def listOfPorts_dict(model):
+def listOfPorts_dict(model: libsbml.Model) -> List[Dict]:
+    """
+    Information dicts for Ports within the plug-in object for packages used by model.
+
+    :param: model: SBML model instance
+    :return: list of info dictionaries for the ports found in the plug-in
+    """
+
     items = []
     model_comp = model.getPlugin("comp")
     if model_comp:
@@ -417,16 +488,29 @@ def listOfPorts_dict(model):
     return items
 
 
-def listOfFunctions_dict(model):
+def listOfFunctions_dict(model: libsbml.Model, math_type: str) -> List[Dict]:
+    """Information dicts for Function definitions within the Model instance.
+
+    :param model: SBML model instance enclosing the function definitions
+    :param math_type: specifies which math rendering mode to use
+    :return: list of info dictionaries for the function definitions in the model
+    """
+
     items = []
     for item in model.getListOfFunctionDefinitions():
         info = infoSbase(item)
-        info["math"] = math(item)
+        info["math"] = math(item, math_type)
         items.append(info)
     return items
 
 
-def listOfUnits_dict(model):
+def listOfUnits_dict(model: libsbml.Model) -> List[Dict]:
+    """Information dicts for Unit definitions within the Model instance.
+
+    :param: model: SBML model instance enclosing the unit definitions
+    :return: list of info dictionaries for the unit definitions in the model
+    """
+
     items = []
     for item in model.getListOfUnitDefinitions():
         info = infoSbase(item)
@@ -437,7 +521,17 @@ def listOfUnits_dict(model):
     return items
 
 
-def listOfCompartments_dict(model, values):
+def listOfCompartments_dict(
+    model: libsbml.Model, values: Dict, math_type: str
+) -> List[Dict]:
+    """Information dicts for Compartments within the Model instance.
+
+    :param model: SBML model instance containing the compartments
+    :param values: dictionary of values
+    :param math_type: specifies which math rendering mode to use
+    :return: list of info dictionaries for the compartments in the model
+    """
+
     items = []
     for item in model.getListOfCompartments():
         info = infoSbase(item)
@@ -452,13 +546,19 @@ def listOfCompartments_dict(model, values):
         if item.isSetSize():
             size = item.size
         else:
-            size = math(values.get(item.id, ""))
+            size = math(values.get(item.id, ""), math_type)
         info["size"] = size
         items.append(info)
     return items
 
 
-def listOfSpecies_dict(model):
+def listOfSpecies_dict(model: libsbml.Model) -> List[Dict]:
+    """Information dicts for Species within the compartments in the Model instance.
+
+    :param: model: SBML model instance containing the species
+    :return: list of info dictionaries for the species in the model
+    """
+
     items = []
     for item in model.getListOfSpecies():
         info = infoSbase(item)
@@ -486,7 +586,7 @@ def listOfSpecies_dict(model):
             cf_value = cf_p.getValue()
             cf_units = cf_p.getUnits()
 
-            info["conversion_factor"] = "{}={} [{}]".format(cf_sid, cf_value, cf_units)
+            info["conversion_factor"] = f"{cf_sid}={cf_value} [{cf_units}]"
         else:
             info["conversion_factor"] = empty_html()
 
@@ -498,16 +598,28 @@ def listOfSpecies_dict(model):
             if sfbc.isSetCharge():
                 c = sfbc.getCharge()
                 if c != 0:
-                    info["fbc_charge"] = "({})".format(sfbc.getCharge())
+                    info["fbc_charge"] = f"({sfbc.getCharge()})"
             if ("fbc_formula" in info) or ("fbc_charge" in info):
-                info["fbc"] = "<br /><code>{} {}</code>".format(
-                    info.get("fbc_formula", ""), info.get("fbc_charge", "")
-                )
+                info[
+                    "fbc"
+                ] = f"""
+                    <br />
+                    <code>
+                        {info.get('fbc_formula', '')}{info.get('fbc_charge', '')}
+                    </code>
+                    """
         items.append(info)
     return items
 
 
-def listOfGeneProducts_dict(model: libsbml.Model):
+def listOfGeneProducts_dict(model: libsbml.Model) -> List[Dict]:
+    """
+    Information dicts for GeneProducts within 'fbc' plug-in object used by the model.
+
+    :param: model: SBML model instance
+    :return: list of info dictionaries for the GeneProducts found in the plug-in object
+    """
+
     items = []
     mfbc = model.getPlugin("fbc")
     if mfbc:
@@ -522,7 +634,17 @@ def listOfGeneProducts_dict(model: libsbml.Model):
     return items
 
 
-def listOfParameters_dict(model, values):
+def listOfParameters_dict(
+    model: libsbml.Model, values: Dict, math_type: str
+) -> List[Dict]:
+    """Information dicts for Parameters defined within the Model instance.
+
+    :param model: SBML model instance using the Parameters
+    :param values: dictionary of values
+    :param math_type: specifies which math rendering mode to use
+    :return: list of info dictionaries for the parameters used in the model
+    """
+
     items = []
     for item in model.getListOfParameters():
         info = infoSbase(item)
@@ -533,11 +655,11 @@ def listOfParameters_dict(model, values):
             value_formula = values.get(item.id, None)
             if value_formula is None:
                 warnings.warn(
-                    "No value for parameter via Value, InitialAssignment or AssignmentRule: {}".format(
-                        item.id
-                    )
+                    f"""No value for parameter via Value, InitialAssignment or
+                    AssignmentRule: {item.id}
+                    """
                 )
-            value = math(value_formula)
+            value = math(value_formula, math_type)
         info["value"] = value
         info["derived_units"] = derived_units(item)
         info["constant"] = boolean(item.constant)
@@ -545,39 +667,66 @@ def listOfParameters_dict(model, values):
     return items
 
 
-def listOfInitialAssignments_dict(model):
+def listOfInitialAssignments_dict(model: libsbml.Model, math_type: str) -> List[Dict]:
+    """Information dicts for Initial Assignments defined within the Model instance.
+
+    :param model: SBML model instance defining the Initial Assignments
+    :param math_type: specifies which math rendering mode to use
+    :return: list of info dictionaries for the initial assignments defined in the model
+    """
+
     items = []
     for item in model.getListOfInitialAssignments():
         info = infoSbase(item)
         info["symbol"] = item.symbol
-        info["assignment"] = math(item)
+        info["assignment"] = math(item, math_type)
         info["derived_units"] = derived_units(item)
         items.append(info)
     return items
 
 
-def listOfRules_dict(model):
+def listOfRules_dict(model: libsbml.Model, math_type: str) -> List[Dict]:
+    """Information dicts for Rules defined within the Model instance.
+
+    :param: model: SBML model instance defining the Rules
+    :return: list of info dictionaries for the rules defined in the model
+    """
+
     items = []
     for item in model.getListOfRules():
         info = infoSbase(item)
         info["variable"] = formating.ruleVariableToString(item)
-        info["assignment"] = math(item)
+        info["assignment"] = math(item, math_type)
         info["derived_units"] = derived_units(item)
 
         items.append(info)
     return items
 
 
-def listOfConstraints_dict(model):
+def listOfConstraints_dict(model: libsbml.Model, math_type: str) -> List[Dict]:
+    """Information dicts for Constraints specified within the Model instance.
+
+    :param model: SBML model instance specified the Constraints
+    :param math_type: specifies which math rendering mode to use
+    :return: list of info dictionaries for the constraits specified in the model
+    """
+
     items = []
     for item in model.getListOfConstraints():
         info = infoSbase(item)
-        info["constraint"] = math(item)
+        info["constraint"] = math(item, math_type)
         items.append(info)
     return items
 
 
-def listOfReactions_dict(model):
+def listOfReactions_dict(model: libsbml.Model, math_type: str) -> List[Dict]:
+    """Information dicts for Reactions occurring in compartments within the model.
+
+    :param model: SBML model instance
+    :param math_type: specifies which math rendering mode to use
+    :return: list of info dictionaries for the reactions found in the model
+    """
+
     items = []
     for item in model.getListOfReactions():
         info = infoSbase(item)
@@ -596,7 +745,7 @@ def listOfReactions_dict(model):
             modifiers = "<br />".join(modifiers)
         info["modifiers"] = modifiers
         klaw = item.getKineticLaw()
-        info["formula"] = math(klaw)
+        info["formula"] = math(klaw, math_type)
         info["derived_units"] = derived_units(klaw)
 
         # fbc
@@ -607,7 +756,13 @@ def listOfReactions_dict(model):
     return items
 
 
-def listOfObjectives_dict(model):
+def listOfObjectives_dict(model: libsbml.Model) -> List[Dict]:
+    """Information dicts for Objective instances defined within the Model instance.
+
+    :param: model: SBML model instance defining the Objectives
+    :return: list of info dictionaries for the objectives defined in the model
+    """
+
     items = []
     mfbc = model.getPlugin("fbc")
     if mfbc:
@@ -622,22 +777,33 @@ def listOfObjectives_dict(model):
                     sign = "-"
                 else:
                     sign = "+"
-                part = "{}{}*{}".format(sign, abs(coefficient), f_obj.getReaction())
+                part = f"{sign}{abs(coefficient)}*{f_obj.getReaction()}"
                 flux_objectives.append(part)
             info["flux_objectives"] = " ".join(flux_objectives)
             items.append(info)
     return items
 
 
-def listOfEvents_dict(model):
+def listOfEvents_dict(model: libsbml.Model, math_type: str) -> List[Dict]:
+    """Information dicts for Events defined within the Model instance.
+
+    :param model: SBML model instance defining the Events
+    :param math_type: specifies which math rendering mode to use
+    :return: list of info dictionaries for the events defined in the model
+    """
+
     items = []
     for item in model.getListOfEvents():
         info = infoSbase(item)
 
         trigger = item.getTrigger()
-        info["trigger"] = "{}<br />initialValue = {}<br />persistent = {}".format(
-            math(trigger), trigger.initial_value, trigger.persistent
-        )
+        info[
+            "trigger"
+        ] = f"""
+            {math(trigger, math_type)}
+            <br />initialValue = {trigger.initial_value}
+            <br /> persistent = {trigger.persistent}
+            """
 
         priority = empty_html()
         if item.isSetPriority():
@@ -650,7 +816,7 @@ def listOfEvents_dict(model):
         info["delay"] = delay
         assignments = ""
         for eva in item.getListOfEventAssignments():
-            assignments += "{} = {}<br />".format(eva.getId(), math(eva))
+            assignments += f"{eva.getId()} = {math(eva, math_type)}<br />"
         if len(assignments) == 0:
             assignments = empty_html()
         info["assignments"] = assignments
@@ -658,62 +824,90 @@ def listOfEvents_dict(model):
     return items
 
 
-##############################
+# -------
 # Helpers
-##############################
-def notes(item):
+# -------
+def notes(item: libsbml.SBase) -> str:
+    """Convert the SBML object's notes subelement to formatted string.
+
+    :param item: SBML object containing the notes subelement
+    :return: formatted string for the notes subelement of the item
+    """
     if item.isSetNotes():
         return formating.notes_to_string(item)
     return ""
 
 
-def cvterm(item):
-    if item.isSetAnnotation():
-        return '<div class="cvterm">{}</div>'.format(formating.annotation_to_html(item))
-    return ""
+def cvterm(item: libsbml.SBase) -> str:
+    """Create HTML code fragment enclosing cvterm data for the item.
 
-
-def sbo(item):
-    if item.getSBOTerm() != -1:
-        return '<div class="cvterm"><a href="{}" target="_blank">{}</a></div>'.format(
-            item.getSBOTermAsURL(), item.getSBOTermID()
-        )
-    return ""
-
-
-def sbaseref(sref):
-    """Formats the SBaseRef
-
-    :param sref:
-    :return:
+    :param item: SBML object for which cvterm data has to be displayed
+    :return: HTML code fragment enclosing cvterm data for the item
     """
-    if sref.isSetPortRef():
-        return "portRef={}".format(sref.getPortRef())
-    elif sref.isSetIdRef():
-        return "idRef={}".format(sref.getIdRef())
-    elif sref.isSetUnitRef():
-        return "unitRef={}".format(sref.getUnitRef())
-    elif sref.isSetMetaIdRef():
-        return "metaIdRef={}".format(sref.getMetaIdRef())
+    if item.isSetAnnotation():
+        return f'<div class="cvterm">{formating.annotation_to_html(item)}</div>'
     return ""
 
 
-def empty_html():
+def sbo(item: libsbml.SBase) -> str:
+    """Create HTML code fragment enclosing SBOTerm data for the item.
+
+    :param item: SBML object for which SBOTerm data has to be displayed
+    :return: HTML code fragment enclosing SBOTerm data for the item
+    """
+
+    if item.getSBOTerm() != -1:
+        return f"""<div class="cvterm">
+                        <a href="{item.getSBOTermAsURL()}" target="_blank">
+                            {item.getSBOTermID()}
+                        </a>
+                    </div>
+                """
+    return ""
+
+
+def sbaseref(sref: libsbml.SBaseRef) -> str:
+    """Format the SBaseRef instance.
+
+    :param sref: SBaseRef instance
+    :return: string containging formatted SBaseRef instance's data
+    """
+
+    if sref.isSetPortRef():
+        return f"portRef={sref.getPortRef()}"
+    elif sref.isSetIdRef():
+        return f"idRef={sref.getIdRef()}"
+    elif sref.isSetUnitRef():
+        return f"unitRef={sref.getUnitRef()}"
+    elif sref.isSetMetaIdRef():
+        return f"metaIdRef={sref.getMetaIdRef()}"
+    return ""
+
+
+def empty_html() -> str:
+    """Create a blank HTML code fragment."""
+
     return '<i class="fa fa-ban gray"></i>'
 
 
-def metaid_html(item):
+def metaid_html(item: libsbml.SBase) -> str:
+    """Create metaid data for the item.
+
+    :param item: SBML object for which metaid data has to be generated
+    :return: HTML code fragment enclosing metaid data for item
+    """
     if item.isSetMetaId():
-        return "<code>{}</code>".format(item.getMetaId())
+        return f"<code>{item.getMetaId()}</code>"
     return ""
 
 
-def id_html(item):
-    """Create info from id and metaid
+def id_html(item: libsbml.SBase) -> str:
+    """Create info from id and metaid.
 
-    :param item:
-    :return:
+    :param item: SBML object for which info is to be generated
+    :return: HTML code fragment enclosing info for item
     """
+
     sid = item.getId()
     meta = metaid_html(item)
 
@@ -721,14 +915,15 @@ def id_html(item):
         display_sid = sid
         if isinstance(item, libsbml.RateRule) and item.isSetVariable():
             display_sid = "d {}/dt".format(item.getVariable())
-        info = '<td id="{}" class="active"><span class="package">{}</span> {}'.format(
-            sid, display_sid, meta
-        )
+        info = f"""
+                <td id="{sid}" class="active">
+                <span class="package">{display_sid}</span> {meta}
+            """
     else:
         if meta:
-            info = '<td class="active">{}'.format(meta)
+            info = f'<td class="active">{meta}'
         else:
-            info = '<td class="active">{}'.format(empty_html())
+            info = f'<td class="active">{empty_html()}'
 
     # create modal information
     info += xml_modal(item)
@@ -736,75 +931,130 @@ def id_html(item):
     return info
 
 
-def annotation_html(item):
+def annotation_html(item: libsbml.SBase) -> str:
+    """Create annotation HTML content for the item.
+
+    :param item: SBML object for which annotation HTML content is to be generated
+    :return: HTML code fragment enclosing annotation for item
+    """
+
     info = '<div class="cvterm">'
     if item.getSBOTerm() != -1:
-        info += '<a href="{}" target="_blank">{}</a><br />'.format(
-            item.getSBOTermAsURL(), item.getSBOTermID()
-        )
+        info += f"""
+            <a href="{item.getSBOTermAsURL()}" target="_blank">
+                {item.getSBOTermID()}
+            </a><br />
+            """
     if item.isSetAnnotation():
         info += formating.annotation_to_html(item)
     info += "</div>"
     return info
 
 
-def math(item):
+def math(item: libsbml.SBase, type: str = "cmathml") -> str:
+    """Create MathML content for the item.
+
+    :param item: SBML object for which MathML content is to be generated
+    :param type: specifies which math rendering mode to use
+    :return: formatted MathML content for the item
+    """
+
     if item:
-        return formating.astnode_to_mathml(item.getMath())
+        math = item.getMath()
+        if type == "cmathml":
+            return formating.astnode_to_mathml(math)
+        elif type == "pmathml":
+            cmathml = formating.astnode_to_mathml(math)
+            return mathml.cmathml_to_pmathml(cmathml)
+        elif type == "latex":
+            latex_str = mathml.astnode_to_latex(math)
+            return f"$${latex_str}$$"
     return empty_html()
 
 
-def boolean(condition):
+def boolean(condition: bool) -> str:
+    """Check the truth value of condition and create corresponding HTML code fragment.
+
+    :param condition: condition for which the truth value is to be checked
+    :return: HTML code fragment
+    """
+
     if condition:
-        return '<td><span class="fas fa-check-circle green"></span><span class="invisible">T</span></td>'
+        return """
+            <td>
+                <span class="fas fa-check-circle green"></span>
+                <span class="invisible">T</span>
+            </td>
+        """
     else:
         return '<td><span class=""><span class="invisible">F</span></span></td>'
 
 
-def annotation_xml(item):
+def annotation_xml(item: libsbml.SBase) -> str:
+    """Create Annotation string for the item.
+
+    :param item: SBML object for which MathML content is to be generated
+    :return: Annotation string for the item
+    """
     if item.isSetAnnotation():
-        return "<pre>{}</pre>".format(item.getAnnotationString().decode("utf-8"))
+        return f"<pre>{item.getAnnotationString().decode('utf-8')}</pre>"
     return ""
 
 
-def xml_modal(item):
-    """Creates modal information for a given sbase.
+def xml_modal(item: libsbml.SBase) -> str:
+    """Create modal information for a given sbase.
 
     This provides some popup which allows to inspect the xml content of the element.
 
-    :param item:
-    :return:
+    :param item: SBML object for which xml content is to be created
+    :return: HTML code fragment enclosing the xml content for the item
     """
-    # filter sbases
+    # filter sbase
     if type(item) is libsbml.Model:
         return ""
 
     hash_id = utils._create_hash_id(item)
 
-    info = """
-      <button type="button" class="btn btn-default btn-xs" data-toggle="modal" data-target="#model-{}"><i class="fa fa-code"></i></button>
-      <div class="modal fade" id="model-{}" role="dialog">
+    info = f"""
+      <button type="button" class="btn btn-default btn-xs" data-toggle="modal" data-target="#model-{hash_id}">
+        <i class="fa fa-code"></i>
+      </button>
+      <div class="modal fade" id="model-{hash_id}" role="dialog">
         <div class="modal-dialog modal-lg">
           <div class="modal-content">
-            <div class="modal-header"><h4 class="modal-title">{}</h4></div>
-            <div class="modal-body"><textarea rows="20" class="form-control" style="min-width: 100%; font-family: 'Courier New'">{}</textarea></div>
+            <div class="modal-header"><h4 class="modal-title">{hash_id}</h4></div>
+            <div class="modal-body">
+                <textarea rows="20" class="form-control" style="min-width: 100%; font-family: 'Courier New'">
+                    {xml(item)}
+                </textarea>
+            </div>
           </div>
         </div>
       </div>
-    """.format(
-        hash_id, hash_id, hash_id, xml(item)
-    )
+    """
     return info
 
 
-def xml(item):
-    html = "{}".format(item.toSBML())
+def xml(item: libsbml.SBase) -> str:
+    """Create SBML specification in XML for the item and return HTML code fragment.
+
+    :param item: SBML object for which SBML specification (in XML) has to be created
+    :return: HTML code fragment enclosing the SBML specification for the item
+    """
+
+    html = f"{item.toSBML()}"
 
     return html
     # return '<textarea style="border:none;">{}</textarea>'.format(item.toSBML())
 
 
-def derived_units(item):
+def derived_units(item: libsbml.SBase) -> str:
+    """Create formatted string for Unit definition object.
+
+    :param item: SBML object from which Unit Definition string is to be created
+    :return: formatted string for Unit Definition derived from the item
+    """
+
     if item:
         return formating.formula_to_mathml(
             formating.unitDefinitionToString(item.getDerivedUnitDefinition())
